# -*- coding: UTF-8 -*-

import numpy as np
import os
import sys
import time
from figure import Figure
from dataset import DataSet
from logger import load_logger
from config import myargparse

frame = "keras"

if frame == "keras":
    from model.model_keras import train, predict
    os.environ["TF_CPP_MIN_LOG_LEVEL"] = '3'
else:
    raise Exception("Wrong frame seletion")


def main():
    parse = myargparse(discription="training config", add_help=False)
    opt = parse.parse_args_and_yaml()
    logger = load_logger(opt)
    try:
        np.random.seed(opt.Model["random_seed"])
        # 在这里获取数据集
        ds = DataSet(opt=opt)
        if opt.do_train:
            train_X, valid_X, train_Y, valid_Y = ds.get_train_and_valid_data(case=2)
            train(opt, [train_X, train_Y, valid_X, valid_Y])
        if opt.do_predict:
            test_X, test_Y, dq_Y = ds.get_test_data(return_label_data=True)
            result = predict(opt, test_X)       # 这里输出的是未还原的归一化预测数据
            fig = Figure(opt, logger, ds)
            fig.draw(test_Y, dq_Y, result)
    except Exception:
        logger.error("Run Error", exc_info=True)


if __name__=="__main__":
    import argparse
    # argparse方便于命令行下输入参数，可以根据需要增加更多
    # parser = argparse.ArgumentParser()
    # parser.add_argument("-t", "--do_train", default=False, type=bool, help="whether to train")
    # parser.add_argument("-p", "--do_predict", default=True, type=bool, help="whether to train")
    # parser.add_argument("-b", "--batch_size", default=64, type=int, help="batch size")
    # parser.add_argument("-e", "--epoch", default=20, type=int, help="epochs num")
    # args = parser.parse_args()

    # con = Config()
    # for key in dir(args):               # dir(args) 函数获得args所有的属性
    #     if not key.startswith("_"):     # 去掉 args 自带属性，比如__name__等
    #         setattr(con, key, getattr(args, key))   # 将属性值赋给Config
    main()

